<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

add_action( 'plugins_loaded','eos_dp_redirect_to_fdp_page' );
// add_action( 'admin_init','eos_dp_redirect_to_fdp_page' );
//Redirect to settings page if it's a FDP page
function eos_dp_redirect_to_fdp_page(){
	if( eos_dp_is_fdp_page() && false !== strpos( $_SERVER['REQUEST_URI'],'/wp-admin/plugins.php' ) ){
		wp_safe_redirect( esc_url( add_query_arg( $_GET,admin_url( 'admin.php' ) ) ) );
	}
}
//Enqueue scripts for back-end
function eos_dp_scripts() {
	global $wp_scripts;
	if( is_object( $wp_scripts ) ){
		$registered = $wp_scripts -> registered;
		foreach( $registered as $handle => $arr ){
			if( 'jquery' !== $handle && false !== strpos( $arr->src,'/plugins/' ) && false === strpos( $arr->src,'freesoul-deactivate-plugins' ) ){
				wp_dequeue_script( $handle );
				wp_deregister_script( $handle );
			}
		}
	}
	wp_deregister_script( 'heartbeat' );
	$params = array( 'is_rtl' => is_rtl() );
	$params['html_url'] = EOS_DP_PLUGIN_URL.'/html/';
	if( isset( $_GET['page'] ) ){
		if( in_array(
			$_GET['page'],
			array(
				'eos_dp_url',
				'eos_dp_admin',
				'eos_dp_firing_order',
				'eos_dp_ajax',
				'eos_dp_by_archive',
				'eos_dp_logged'
			) )
		){
			$params['page'] = esc_js( esc_attr( $_GET['page'] ) );
			wp_enqueue_script( 'jquery-ui-sortable',array( 'jquery' ) );
		}
		if( in_array( $_GET['page'],array( 'eos_dp_menu','eos_dp_by_archive','eos_dp_by_term_archive' ) ) ){
			wp_enqueue_script( 'jquery-ui-draggable',array( 'jquery' ) );
		}
	}
	wp_enqueue_script( 'eos-dp-backend',EOS_DP_PLUGIN_URL.'/admin/js/fdp-admin-8.js', array( 'jquery' ),EOS_DP_VERSION );
	wp_localize_script( 'eos-dp-backend','eos_dp_js',$params );
}
//Enqueue style for back-end
function eos_dp_style() {
	$action = false;
	if( function_exists( 'get_current_screen' ) ){
		$screen = get_current_screen();
		$action = isset( $screen->action ) ? $screen->action : false;
	}
	if(
		( $action && $action === 'add' )
		|| ( isset( $_GET['post'] )
		&& isset( $_GET['action'] )
		&& $_GET['action'] === 'edit' )
		|| eos_dp_is_fdp_page()
		){
		$rtl = is_rtl() ? '-rtl' : '';
		wp_enqueue_style( 'eos-dp-admin-style',EOS_DP_PLUGIN_URL.'/admin/css/fdp-admin-8'.$rtl.'.css',array(),EOS_DP_VERSION );
	}
}

add_action( 'admin_head','eos_dp_admin_inline_style' );
//Add inline style on admin pages
function eos_dp_admin_inline_style(){
	echo '<style>.eos_dp_plugin_upgrade_notice+p:before{content:"";display:none;opacity:0}</style>';
}

if( !function_exists( 'eos_dp_get_option' ) ){
	//Get options in case of single or multisite installation.
	function eos_dp_get_option( $option ){
		if( !is_multisite() ){
			return get_option( $option );
		}
		else{
			return get_blog_option( get_current_blog_id(),$option );
		}
	}
}
//It adds a settings link to the action links in the plugins page
function eos_dp_plugin_add_settings_link( $links ) {
    $settings_link = '<a class="eos-dp-setts" href="'.admin_url( 'admin.php?page=eos_dp_by_post_type' ).'">' . __( 'Settings','eos-dp' ). '</a>';
    array_push( $links, $settings_link );
    $docu_link = '<a class="eos-dp-help" href="https://freesoul-deactivate-plugins.com/documentation/" target="_blank" rel="noopener">' . __( 'Documentation','eos-dp' ). '</a>';
    array_push( $links, $docu_link );
    $support_link = '<a class="eos-dp-help" href="https://wordpress.org/support/plugin/freesoul-deactivate-plugins/" target="_blank" rel="noopener">' . __( 'Support','eos-dp' ). '</a>';
    array_push( $links, $support_link );
  	return $links;
}
//It redirects to the plugin settings page on successfully plugin activation
function eos_dp_redirect_to_settings(){
	if( get_transient( 'freesoul-dp-notice-succ' ) ){
		delete_transient( 'freesoul-dp-notice-succ' );
		if( !get_transient( 'freesoul-dp-updating-mu' ) ){
			wp_safe_redirect( admin_url( 'admin.php?page=eos_dp_by_post_type' ) );
		}
	}
	if( isset( $_REQUEST['eos_dp_activated_from'] ) ){
		wp_safe_redirect( add_query_arg( 'page',esc_attr( $_REQUEST['eos_dp_activated_from'] ),admin_url( 'admin.php' ) ) );
	}
	$previous_version = eos_dp_get_option( 'eos_dp_version' );
	$version_compare = version_compare( $previous_version, EOS_DP_VERSION,'<' );
	if( $version_compare && EOS_DP_NEED_UPDATE_MU ){
		//if the plugin was updated and we need to update also the mu-plugin
		define( 'EOS_DP_DOING_MU_UPDATE',true );
		if( file_exists( WPMU_PLUGIN_DIR.'/eos-deactivate-plugins.php' ) ){
			unlink( WPMU_PLUGIN_DIR.'/eos-deactivate-plugins.php' );
		}
		require EOS_DP_PLUGIN_DIR.'/plugin-activation.php';
		eos_dp_update_option( 'eos_dp_version',EOS_DP_VERSION );
		set_transient( 'freesoul-dp-updating-mu',5 );
	}
}
//It creates the transient needed for displaing plugin notices after activation
function eos_dp_admin_notices(){
	//It creates the transient needed for displaing plugin notices after activation
	if( get_transient( 'freesoul-dp-notice-fail' ) ){
		delete_transient( 'freesoul-dp-notice-fail' );
	?>
	<div class="notice notice-error is-dismissible">
		<p><?php _e( 'You have no direct write access, Freesoul Deactivate Plugins was not able to create the necessary mu-plugin and will not work.', 'eos-dp' ); ?></p>
	</div>
	<?php
	}
	$mu_exists = file_exists( WPMU_PLUGIN_DIR.'/eos-deactivate-plugins.php' );
	$message = $class = '';
	if( eos_dp_is_fdp_page() && ( !defined( 'EOS_DP_MU_VERSION' ) || EOS_DP_MU_VERSION !== EOS_DP_VERSION || !$mu_exists ) ){
		if( !$mu_exists ){
			$class = 'error';
			$message = '<p><h1>'.sprintf( __( "Very important file missing. First, refresh this page, if you still see this message, disable Freesoul Deactivate Plugins and activate it again. If nothing helps, copy the file %s and put it into the directory %s","eos-dp" ),'/wp-content/plugins/freesoul-deactivate-plugins/mu-plugins/eos-deactivate-plugins.php','wp-content/mu-plugins/' ).'</h1></p>';
		}
		elseif( $mu_exists && !defined( 'EOS_DP_MU_VERSION' ) ){
			$class = 'error';
			$message = '<p><h1>'.sprintf( __( "Issue detected. It looks the file %s has been modified.","eos-dp" ),WPMU_PLUGIN_DIR.'/eos-deactivate-plugins.php',WPMU_PLUGIN_DIR ).'</h1></p>';
		}
		elseif( defined( 'EOS_DP_MU_VERSION' ) && EOS_DP_MU_VERSION !== EOS_DP_VERSION ){
			$class = 'warning';
			$message = '<p>'.__( 'Issue detected. Disable Freesoul Deactivate Plugins and then activate it again.','eos-dp' ).'</p>';
			$message .= '<p>'.sprintf( __( "If you still see this message after disabling and reactivating Freesoul Deactivate Plugins and after refreshing this page, open a thread on the %sPlugin Support Forum%s","eos-dp" ).'</p>','<a href="https://wordpress.org/support/plugin/freesoul-deactivate-plugins/" target="_blank" rel="noopener">','</a>' ).'</p>';
		}
		?>
		<div class="notice notice-<?php echo $class; ?> is-dismissible" style="line-height:1.5">
			<?php echo wp_kses( $message,array( 'h1' => array(),'p' => array(),'a' => array( 'href' => array(),'rel' => array(),'target' => array() ) ) ); ?>
		</div>
		<?php
	}
}
//It adds the plugin setting page under plugins menu
function eos_dp_options_page(){
	if( !apply_filters( 'eos_dp_user_can_settings',true ) ) return;
	$capability = apply_filters( 'eos_dp_settings_capability','activate_plugins' );
	add_menu_page( __( 'Freesoul Deactivate Plugins','eos-dp' ),__( 'Freesoul Deactivate Plugins','eos-dp' ),$capability,'eos_dp_by_post_type','eos_dp_by_post_type_callback','dashicons-plugins-checked',65 );
	add_plugins_page( __( 'Freesoul Deactivate Plugins','eos-dp' ),__( 'Freesoul Deactivate Plugins','eos-dp' ),$capability,'eos_dp_by_post_type','eos_dp_by_post_type_callback',20 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate by post type','eos-dp' ),__( 'Post Types','eos-dp' ),$capability,'eos_dp_by_post_type','eos_dp_by_post_type_callback',20 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate on each single post','eos-dp' ),__( 'Singles','eos-dp' ),$capability,'eos_dp_menu','eos_dp_options_page_callback',20 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate by Archive','eos-dp' ),__( 'Archives','eos-dp' ),$capability,'eos_dp_by_archive','eos_dp_by_archive_callback',30 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate by Term Archive','eos-dp' ),__( 'Term Archives','eos-dp' ),$capability,'eos_dp_by_term_archive','eos_dp_by_term_archive_callback',40 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate on mobile devices','eos-dp' ),__( 'Mobile','eos-dp' ),$capability,'eos_dp_mobile','eos_dp_mobile_callback',50 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate on search resutls page','eos-dp' ),__( 'Search','eos-dp' ),$capability,'eos_dp_search','eos_dp_search_callback',60 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate by URL','eos-dp' ),__( 'Custom URLs','eos-dp' ),$capability,'eos_dp_url','eos_dp_by_url_callback',70 );
	add_submenu_page( null,__( 'Deactivate in Administration Pages by custom URLs','eos-dp' ),__( 'Backend URLs','eos-dp' ),$capability,'eos_dp_admin_url','eos_dp_by_admin_url_callback',80 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate in Administration Pages','eos-dp' ),__( 'Backend','eos-dp' ),$capability,'eos_dp_admin','eos_dp_admin_callback',90 );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Deactivate depending on third party plugin action','eos-dp' ),__( 'Actions','eos-dp' ),$capability,'eos_dp_integration','eos_dp_integration_callback',100 );
	add_submenu_page( null,__( 'Firing Order','eos-dp' ),__( 'Firing Order','eos-dp' ),$capability,'eos_dp_firing_order','eos_dp_firing_order_callback',110 );
	add_submenu_page( null,esc_html__( 'Roles Manager','eos-dp' ),esc_html__( 'Roles Manager','eos-dp' ),'manage_options','eos_dp_roles_manager','eos_dp_pro_roles_manager_callback',120 );
	add_submenu_page( null,__( 'Testing','eos-dp' ),__( 'Testing','eos-dp' ),$capability,'eos_dp_smoke_tests','eos_dp_smoke_tests_callback',130 );
	add_submenu_page( null,__( 'Create plugin','eos-dp' ),__( 'Create plugin','eos-dp' ),$capability,'eos_dp_create_plugin','eos_dp_create_plugin_callback',140 );
	add_submenu_page( null,__( 'Favorite plugins','eos-dp' ),__( 'Favorite plugins','eos-dp' ),$capability,'eos_dp_favorite_plugins','eos_dp_favorite_plugins_callback',150 );
	do_action( 'eos_dp_admin_menu_items' );
	add_submenu_page( 'eos_dp_by_post_type',__( 'Help','eos-dp' ),__( 'Help','eos-dp' ),$capability,'eos_dp_help','eos_dp_help_callback',200 );
}
//It displays the ajax loader gif
function eos_dp_ajax_loader_img(){
	?>
	<img alt="<?php _e( 'Ajax loader','eos-dp' ); ?>" class="ajax-loader-img eos-not-visible" width="30" height="30" src="<?php echo EOS_DP_PLUGIN_URL; ?>/img/ajax-loader.gif" />
	<?php
}
function eos_dp_alert_plain_permalink(){
	$permalink_structure = basename( get_option( 'permalink_structure' ) );
	if( false === strpos( $permalink_structure,'%postname%' )  ){
		$permalinks_label = __( 'the actual permalinks structure is not supported','eos-dp' );
		if( '' === $permalink_structure ){
			$permalinks_label = __( 'the permalinks are set as plain','eos-dp' );
		}
		elseif( '/archives/%post_id%' === $permalink_structure ){
			$permalinks_label = __( 'the permalinks are set as numeric','eos-dp' );
		}
	?>
	<div id="eos-dp-plain-permalink-wrg" style="line-height:1;margin:20px 0;padding:10px;color:#23282d;background:#fff;border-<?php echo is_rtl() ? 'right' : 'left'; ?>:4px solid  #dc3232">
		<div>
			<h1><?php printf( __( 'No plugins will be permanently deactivated because %s.','eos-dp' ),$permalinks_label ); ?></h1>
			<h1><?php _e( 'Only the permalinks structures "Day and name", "Month and name", "Post name"  and the custom ones ending with "%postname%" are supported (they are also better for SEO).','eos-dp' ); ?></h1>
		</div>
		<div>
			<a class="button" target="_blank" href="<?php echo admin_url( 'options-permalink.php' ); ?>"><?php  _e( 'Change Permalinks Structure','eos-dp' ); ?></a>
		</div>
	</div>
	<?php
	}
}
//It adds the plugin names orientation controll
function eos_dp_plugin_names_orientation_ctrl(){
	?>
	<div id="eos-dp-orientation-wrp">
		<span id="eos-dp-names-orientation"><?php _e( 'Plugin names orientation','eos-dp' ); ?></span>
		<span id="eos-dp-orientation-icon">
			<span></span>
		</span>
	</div>
	<div id="eos-dp-fit-to-screen" class="right" style="margin-top:22px">
		<span style="position:relative;top:-9px"><?php _e( 'Zoom','eos-dp' ); ?></span><span title="<?php _e( 'Zoom','eos-dp' ); ?>" class="hover dashicons dashicons-search"></span>
	</div>
	<?php
}
//It gets the plugins that are active/deactive for each post type.
function eos_dp_post_types_empty(){
	if( isset( $_POST['eos_dp_setts'] ) ){
		$setts = $_POST['eos_dp_setts'];
		if( isset( $setts['eos_dp_post_types'] ) ){
			$post_types = json_decode( str_replace( '\\','',$setts['eos_dp_post_types'] ),true );
		}
		else{
			$post_types = get_post_types( array( 'publicly_queryable' => true,'public' => true ),'names','or' );
		}
		$post_types = array_map( 'esc_attr',$post_types );
	}
	else{
		$post_types = get_post_types( array( 'publicly_queryable' => true,'public' => true ),'names','or' );
	}
	return array_fill_keys(
				$post_types,
			array(
				'1',
				implode( ',',array_fill( 0,count( array_unique( get_option( 'active_plugins',array() ) ) ),'' ) )
			)
		);
}
//It returns the active plugins excluding Freesoul Deactivate Plugins
function eos_dp_active_plugins(){
	$active = array_unique( get_option( 'active_plugins',array() ) );
	unset( $active[array_search( EOS_DP_PLUGIN_BASE_NAME,$active )] );
	if( defined( 'EOS_DP_PRO_PLUGIN_BASE_NAME' ) ){
		unset( $active[array_search( EOS_DP_PRO_PLUGIN_BASE_NAME,$active )] );
	}
	$active = array_values( $active );
	$n = 0;
	foreach( $active as $v ){
		if( false === strpos( $v,'/' ) ){
			unset( $active[$n] );
		}
		++$n;
	}
	return array_values( $active );
}
function eos_dp_get_plugins() {
	$plugin_root = WP_PLUGIN_DIR;
	// Files in wp-content/plugins directory
	$plugins_dir = @ opendir( $plugin_root);
	$plugin_files = array();
	if ( $plugins_dir ) {
		while (($file = readdir( $plugins_dir ) ) !== false ) {
			if ( substr($file, 0, 1) == '.' || strpos( '_'.$file,'freesoul-deactivate-plugins' ) > 0 ) continue;
			if ( is_dir( $plugin_root.'/'.$file ) ) {
				$plugins_subdir = @ opendir( $plugin_root.'/'.$file );
				if ( $plugins_subdir ) {
					while (($subfile = readdir( $plugins_subdir ) ) !== false ) {
							if ( substr($subfile, 0, 1) == '.' )
									continue;
							if ( substr($subfile, -4) == '.php' )
									$plugin_files[] = "$file/$subfile";
					}
					closedir( $plugins_subdir );
				}
			}
			else {
				if( substr($file, -4) == '.php' ) $plugin_files[] = $file;
			}
		}
		closedir( $plugins_dir );
	}
	if ( empty( $plugin_files ) ) return array();
	foreach ( $plugin_files as $plugin_file ) {
		if ( !is_readable( "$plugin_root/$plugin_file" ) ) continue;
		$plugins[plugin_basename( $plugin_file )] = 1;
	}
	uasort( $plugins,'eos_dp_sort_uname_callback' );
	return $plugins;
}

//Callback to sort array by a 'Name' key.
function eos_dp_sort_uname_callback( $a, $b ) {
	if( !is_array( $a ) || !is_array( $b ) ) return false;
	return strnatcasecmp( $a['Name'], $b['Name'] );
}

//It returns the updated plugins table after a third plugin activation
function eos_dp_get_updated_plugins_table(){
	$plugins_table = eos_dp_get_option( 'eos_post_types_plugins' );
	if( !$plugins_table || !is_array( $plugins_table ) || empty( $plugins_table ) ) return eos_dp_post_types_empty();
	if( 'activated' !== eos_dp_get_option( 'eos_dp_new_plugin_activated' ) ){
		$plugins_table = !empty( $plugins_table ) ? $plugins_table : eos_dp_post_types_empty();
	}
	else{
		$old_post_types = array_keys( $plugins_table );
		$new_post_types = get_post_types( array( 'publicly_queryable' => true ) );
		if( isset( $new_post_types['attachment'] ) ){
			unset( $new_post_types['attachment'] );
		}
		$new_post_types = array_keys( array_merge( array( 'page' => 'page' ),$new_post_types ) );
		if( $old_post_types !== $new_post_types ){
			foreach( $new_post_types as $key ){
				if( !isset( $plugins_table[$key] ) ){
					$plugins_table[$key] = array(
						'1',
						implode( ',',array_fill( 0,count( array_unique( get_option( 'active_plugins',array() ) ) ),'' ) )
					);
				}
			}
		}
	}
	return $plugins_table;
}

//It returns the important pages
function eos_dp_important_pages(){
	$locations = get_nav_menu_locations();
	$ids = $woo_ids = $nav_ids = array();
	foreach( $locations as $location ){
		$menuItems = wp_get_nav_menu_items( $location );
		if( is_array( $menuItems ) ){
			foreach($menuItems as $page) {
				$ids[] = $page->object_id;
				$nav_ids[] = $page->object_id;
			}
		}
	}
	$keys = array(
		'comingsoon_input_page',
		'woocommerce_shop_page_id',
		'woocommerce_cart_page_id',
		'woocommerce_checkout_page_id',
		'woocommerce_pay_page_id',
		'woocommerce_thanks_page_id',
		'woocommerce_myaccount_page_id',
		'woocommerce_edit_address_page_id',
		'woocommerce_view_order_page_id',
		'woocommerce_terms_page_id',
		'wp_page_for_privacy_policy'
	);
	if( 'page' === eos_dp_get_option( 'show_on_front' ) ){
		$keys[] = 'page_for_posts';
	}
	elseif( 'posts' === eos_dp_get_option( 'show_on_front' ) ){
		$keys[] = 'page_on_front';
	}
	foreach( $keys as $opt_key ){
		$id = eos_dp_get_option( $opt_key );
		if( $id ){
			$ids[]= $id;
		}
		if( false !== strpos( $opt_key,'woocommerce' ) ){
			$woo_ids[] = $id;
		}
	}
	$sticky_posts = eos_dp_get_option( 'sticky_posts' );
	if( $sticky_posts ){
		$ids = array_merge( $ids,$sticky_posts );
	}
	return array(
		'ids' => array_values( array_unique( $ids ) ),
		'woo_ids' => array_values( array_unique( $woo_ids ) ),
		'nav_ids' => array_values( array_unique( $nav_ids ) ),
	);
}

add_action( 'in_plugin_update_message-freesoul-deactivate-plugins/freesoul-deactivate-plugins.php','eos_dp_get_update_notice' );
//Check if it's a major release and return the upgrade notice
function eos_dp_get_update_notice(){
	$transient_name = 'eos_dp_changelog_'.EOS_DP_VERSION;
	$upgrade_notice = get_transient( $transient_name );
	if ( false === $upgrade_notice || '' === $upgrade_notice ) {
		$response = wp_safe_remote_get( 'https://plugins.svn.wordpress.org/freesoul-deactivate-plugins/trunk/readme.txt' );
		if ( !is_wp_error( $response ) && ! empty( $response['body'] ) ) {
			$upgrade_notice = substr( $response['body'],strpos( $response['body'],'== Changelog ==' ) + 15 );
			$arr = explode( '==',$upgrade_notice );
			$upgrade_notice = $arr[0];
			$upgrade_notice = wp_kses_post( $upgrade_notice );
			set_transient( $transient_name, $upgrade_notice,3600*24 );
		}
	}
	$warning = '';
	$support = '';
	$info = eos_dp_get_plugin_info( 'freesoul-deactivate-plugins' );
	if( $info ){
		$warning .= 'Make always a backup before updating any plugin.';
		$warning .= '<br/>';
		$support .= '<br/>';
		$support .= '<br/>';
		$support .= sprintf( '<b>Activities on the Support Forum.</b>','<a href="https://wordpress.org/support/plugin/freesoul-deactivate-plugins/" target="_blan" rel="noopener">','</a>' );;
		$support .= '<br/>';
		$support .= sprintf( 'Threads opened in the last two months: %s',esc_attr( $info['threads'] ) );
		$support .= '<br/>';
		$support .= sprintf( 'Issued resolved in the last two months: %s',esc_attr( $info['solved_threads'] ) );
		$support .= '<br/>';
		$support .= '<br/>';
		$support .= sprintf( 'If you have any issues, don\'t hesitate to open a thread on the %sSupport Forum%s','<a href="https://wordpress.org/support/plugin/freesoul-deactivate-plugins/" target="_blan" rel="noopener">','</a>' );
		$support .= '<br/>';
	}
	echo '<div class="eos_dp_plugin_upgrade_notice"><br/>'.$warning.'<br/><b>Last changes:</b><br/>'.wp_kses_post( str_replace( '*','</br>',$upgrade_notice ) ).$support.'</div>';
}

if( isset( $_GET['eos_dp_info'] ) && 'true' === $_GET['eos_dp_info'] ){
	add_action( 'install_plugins_pre_plugin-information','eos_dp_plugin_information' );
}
//Add plugin information if it's not on the repository
function eos_dp_plugin_information(){
	if( isset( $_GET['eos_dp'] ) ){
		$api = plugins_api(
			'plugin_information',
			array(
				'slug' => wp_unslash( $_REQUEST['plugin'] ),
			)
		);
		if ( is_wp_error( $api ) ) {
			$plugin_data = get_plugin_data( WP_PLUGIN_DIR.'/'.$_GET['eos_dp'] );
			if( $plugin_data ){
				global $plugin_data;
				remove_all_actions('admin_notices');
				remove_all_actions('all_admin_notices');
				require_once  ABSPATH.'/wp-admin/admin-header.php';
				add_action( 'admin_footer','eos_dp_plugin_page_content' );

			}
		}
		else{
			add_action( 'admin_footer','eos_dp_plugin_badge' );
		}
	}
}
//Plugin page content
function eos_dp_plugin_page_content(){
	global $plugin_data;
	?>
	</div>
	<div id="plugin-information" style="position:fixed;width:100vw;left:0;right:0;background:#fff;z-index:9999;height:100vh">
		<div id="plugin-information-title" class="with-banner">
			<div class="vignette"></div>
			<h2><?php echo esc_html( $plugin_data['Name'] ); ?></h2>
		</div>
		<div id="plugin-information-tabs" class="with-banner">
			<a class="current" href="#"><?php _e( 'Description','eos-dp' ); ?></a>
		</div>
		<div class="fyi" style="min-width:240px">
			<ul>
				<?php
				foreach( $plugin_data as $key => $value ){
					if( !$value || '' === $value || in_array( $key,array( 'Name','Description','PluginURI','TextDomain' ) ) ) continue;
				?>
				<li><strong><?php echo esc_html( $key ); ?>:</strong> <?php echo wp_kses( $value,array( 'a' => array( 'href' => array() ) ) ); ?></li>
				<?php } ?>
			</ul>
		</div>
		<div id="section-holder">
			<div id="section-description" style="padding:20px" class="section">
				<?php if( isset( $plugin_data['Author'] ) ){ ?>
				<div style="margin-top:64px">
					<p><?php printf( __( 'By %s','eos-dp' ),wp_kses( $plugin_data['Author'],array( 'a' => array( 'href' => array() ) ) ) ); ?></p>
				</div>
				<?php } ?>
				<?php if( isset( $plugin_data['Description'] ) ){ ?>
				<div style="margin-top:64px">
					<p><?php echo wp_kses( $plugin_data['Description'],array( 'a' => array( 'href' => array() ) ) ); ?></p>
				</div>
				<?php
				}
				if( isset( $plugin_data['PluginURI'] ) ){
				?>
				<div style="margin-top:32px">
					<p><?php printf( __( 'More info at %s','eos-dp' ),'<a href="'.esc_url( $plugin_data['PluginURI'] ).'">'.esc_url( $plugin_data['PluginURI'] ).'</a>' ); ?></p>
				</div>
			</div>
		</div>
	</div>
	<div class="hidden">
	<?php
	}
}

//Plugin badge
function eos_dp_plugin_badge(){
	?>
	</div>
	<div id="eos-dp-plugin-badge" style="position:fixed;top:10px;padding:10px;z-index:99999">
		<p>
			<a target="_blank" rel="noopener" href="https://plugintests.com/plugins/<?php echo esc_attr( $_GET['plugin'] ); ?>/latest"><img src="https://plugintests.com/plugins/<?php echo esc_attr( $_GET['plugin'] ); ?>/php-badge.svg"></a>
		</p>
		<p>
			<a class="button" target="_blank" rel="noopener" href="https://plugintests.com/plugins/<?php echo esc_attr( $_GET['plugin'] ); ?>/latest"><?php _e( 'Go to the last plugin test results','eos-dp' ); ?></a>
		</p>
	</div>
	<div class="hidden">
	<?php
}
//Return list of installed themes
function eos_dp_active_themes_list( $dummy_html = true ){
	$active_themes = wp_get_themes();
	if( count ( $active_themes ) < 1 ) return false;
	$output = '<select class="eos-dp-themes-list">';
	$output .= '<option value="false">'.__( 'Current Theme','eos-dp' ).'</option>';
	foreach ( $active_themes as $theme => $v ){
		$output .= '<option value="'.esc_attr( $theme ).'">'.esc_html( $theme ).'</option>';
	}
	$output .= '<option value="empty_theme">'.__( 'Empty Theme','eos-dp' ).'</option>';
	if( $dummy_html ){
		$output .= '<option value="dummy_html">'.__( 'Dummy HTML file','eos-dp' ).'</option>';
	}
	$output .= '</select>';
	return $output;
}

//Return true if it's a FDP settings page, or false if not socket_accept
function eos_dp_is_fdp_page(){
	return isset( $_GET['page'] )
	&& in_array( $_GET['page'],
		array(
			'eos_dp_admin_url',
			'eos_dp_admin',
			'eos_dp_advanced_support',
			'eos_dp_ajax',
			'eos_dp_by_archive',
			'eos_dp_by_post_type',
			'eos_dp_by_term_archive',
			'eos_dp_create_plugin',
			'eos_dp_favorite_plugins',
			'eos_dp_firing_order',
			'eos_dp_help',
			'eos_dp_hooks',
			'eos_dp_integration',
			'eos_dp_logged',
			'eos_dp_menu',
			'eos_dp_mobile',
			'eos_dp_pro_bulk_actions',
			'eos_dp_pro_import_export',
			'eos_dp_pro_installations',
			'eos_dp_pro_license',
			'eos_dp_pro_settings',
			'eos_dp_report',
			'eos_dp_roles_manager',
			'eos_dp_search',
			'eos_dp_smoke_tests',
			'eos_dp_status',
			'eos_dp_testing',
			'eos_dp_url'
		) );
}

add_action( 'admin_head','eos_dp_add_admin_inline_style' );
function eos_dp_add_admin_inline_style(){
	?>
	<style type="text/css">
	#wpadminbar .eos-dp-top-admin-icon{
		height:20px;
		width:20px;
		line-height: 0;
		margin-right: 4px;
		display: inline-block;
		vertical-align: middle
	}
	.rtl #wpadminbar .eos-dp-top-admin-icon{
		margin-right:0;
		margin-left:4px
	}
	#wp-admin-bar-eos-dp-menu,
	#wp-admin-bar-eos-dp-menu li{
		opacity:1
	}
	#eos-dp-ab-ico-editphppost_typeproduct
	</style>
	<?php
}
add_action( 'admin_bar_menu','eos_dp_light_admin_menu',40 );
// Add ligth admin menu to admin top  bar
function eos_dp_light_admin_menu( $wp_admin_bar ){
	$opts = eos_dp_get_option( 'eos_dp_general_setts' );
	if( true || !eos_dp_is_fdp_page() && $opts && isset( $opts['replace_admin_menu'] ) && 'false' !== $opts['replace_admin_menu'] ){
		if( isset( $opts['admin_menu'] ) && isset( $opts['admin_submenu'] ) ){
			$params['admin_url'] = admin_url();
			$params['nonce'] = wp_create_nonce( 'eos_dp_create_admin_menu' );
			wp_enqueue_script( 'eos-dp-admin-menu',EOS_DP_PLUGIN_URL.'/admin/js/fdp-menu.js',array( 'jquery' ),EOS_DP_VERSION,true );
			wp_localize_script( 'eos-dp-admin-menu','eos_dp_menu',$params );
		}
	}
	if( !current_user_can( 'manage_options' ) ) return $wp_admin_bar;
	if( $opts && isset( $opts['menu_in_topbar'] ) && 'false' === $opts['menu_in_topbar'] ) return $wp_admin_bar;
	$admin_menus = eos_dp_get_option( 'eos_dp_admin_menu' );
	if( !$admin_menus ) return $wp_admin_bar;
	$admin_menus = json_decode( stripslashes( $admin_menus ),true );
	if( !$admin_menus || !is_array( $admin_menus ) || empty( $admin_menus ) ) return $wp_admin_bar;
	$wp_admin_bar->add_menu( array(
		'id'    => 'eos-dp-menu',
		'title' => '<span class="dashicons dashicons-plugins-checked" style="font-family:dashicons"></span> '.__( 'Admin Menu','eos-dp' ),
	));
	$wp_admin_bar->add_node( array(
		'parent' => 'eos-dp-menu',
		'id'     => 'eos-dp-fdp',
		'title'  => '<span class="eos-dp-top-admin-icon dashicons-before dashicons-plugins-checked"></span><span>'.__( 'Freesoul Deactivate Plugins','eos-dp' ).'</span>',
		'href' => admin_url( 'admin.php?page=eos_dp_by_post_type' )
	) );
	$wp_admin_bar->add_node( array(
		'parent' => 'eos-dp-menu',
		'id'     => 'eos-dp-menu-item-separator-1',
		'title' => '__________________________________'
	) );
	$n = 0;
	$k = 0;
	foreach( $admin_menus as $admin_menu => $arr ){
		$args = array(
			'parent' => 'eos-dp-menu',
			'id'     => 'eos-dp-parent-menu-item-'.$n,
			'title'  => isset( $arr['parent_title'] ) ? esc_html( $arr['parent_title'] ) : esc_html( $arr['title'] )
		);
		if( false !== strpos( $admin_menu,'wc-' ) ){
			$arr['icons'] = 'dashicons-cart';
		}
		if( isset( $arr['icons'] ) && '' !== $arr['icons'] && false === strpos( $arr['icons'],'http' ) ){
			$args['title'] = '<span id="eos-dp-ab-ico-'.sanitize_key( $admin_menu ).'" class="eos-dp-top-admin-icon dashicons-before '.esc_attr( $arr['icons'] ).'"></span><span>'.$args['title'].'</span>';
		}
		else{
			$args['title'] = '<span id="eos-dp-ab-ico-'.sanitize_key( $admin_menu ).'" class="eos-dp-top-admin-icon dashicons-before"></span><span>'.$args['title'].'</span>';
		}
		$wp_admin_bar->add_node( $args );
		$submenu = $arr['submenu'];
		foreach( $submenu as $arr2 ){
			$wp_admin_bar->add_node( array(
				'parent' => 'eos-dp-parent-menu-item-'.$n,
				'id'     => 'eos-dp-sub-menu-item-'.$k,
				'title'  => esc_html( $arr2['title'] ),
				'href'  => esc_url( $arr2['url'] )
			) );
			++$k;
		}
		++$n;
	}
	$wp_admin_bar->add_node( array(
		'parent' => 'eos-dp-menu',
		'id'     => 'eos-dp-menu-item-separator-2',
		'title' => '__________________________________'
	) );
	$wp_admin_bar->add_node( array(
		'parent' => 'eos-dp-menu',
		'id'     => 'eos-dp-menu-item-backend-settings',
		'title' => '<span class="dashicons dashicons-admin-generic" style="line-height:27px;font-family:dashicons"></span> '.__( 'Backend Settings','eos-dp' ),
		'href' => admin_url( 'admin.php?page=eos_dp_admin' )
	) );
	return $wp_admin_bar;
}

//Return option as array
function eos_dp_get_option_array( $option ){
	$opts = eos_dp_get_option( 'eos_dp_general_setts' );
	if( !$opts || !is_array( $opts ) ){
		$opts = array();
	}
	return $opts;
}

//Get plugin information
function eos_dp_get_plugin_info( $slug ){
	$response = wp_remote_get( 'https://api.wordpress.org/plugins/info/1.0/freesoul-deactivate-plugins.json' );
	if( !is_wp_error( $response ) ){
		$json = wp_remote_retrieve_body( $response );
		$info = json_decode( $json );
		$return = array();
		if( is_object( $info ) ){
			$return['threads'] = $info->support_threads;
			$return['solved_threads'] = $info->support_threads_resolved;
			$return['last_updated'] = $info->last_updated;
			$return['versions'] = $info->versions;
			return $return;
		}
	}
	return false;
}

//Check if the input is a plugin pathinfo
function eos_dp_is_not_empty_string( $string ){
	return '' !== $string ? '0' : '1';
}

//Return $plugins_table
function eos_dp_plugins_table(){
	$plugins_table = eos_dp_get_updated_plugins_table();
	$plugins_table = is_array( $plugins_table ) && !empty( $plugins_table ) ? $plugins_table : eos_dp_post_types_empty();
	return $plugins_table;
}


//Returns array with information about third plugins if a plugin for integration is active, false if no plugiin is found
function eos_dp_plugins_integration(){
	$plugins = array(
		'wordpress-core' => array(
			'is_active' => true,
			'ajax_actions' => array(
				'custom-background-add' => array( 'description' => __( 'Add custom background','eos-dp' ) ),
				'set-background-image' => array( 'description' => __( 'Set background image','eos-dp' ) ),
				'custom-header-crop' => array( 'description' => __( 'Crop custom header','eos-dp' ) ),
				'custom-header-add' => array( 'description' => __( 'Add custom header','eos-dp' ) ),
				'custom-header-remove' => array( 'description' => __( 'Remove custom header','eos-dp' ) ),
				'customize_save' => array( 'description' => __( 'Save customize','eos-dp' ) ),
				'customize_trash' => array( 'description' => __( 'Trash customize','eos-dp' ) ),
				'customize_refresh_nonces' => array( 'description' => __( 'Refresh nonces customize','eos-dp' ) ),
				'customize_load_themes' => array( 'description' => __( 'Load themes customize','eos-dp' ) ),
				'customize_override_changeset_lock' => array( 'description' => __( 'Override changeset lock customize','eos-dp' ) ),
				'customize_dismiss_autosave_or_lock' => array( 'description' => __( 'Dismiss autosave or lock customize','eos-dp' ) ),
				'load-available-menu-items-customizer' => array( 'description' => __( 'Load available menu items customize','eos-dp' ) ),
				'search-available-menu-items-customizer' => array( 'description' => __( 'Search available menu items customize','eos-dp' ) ),
				'customize-nav-menus-insert-auto-draft' => array( 'description' => __( 'Nav menus insert auto-draft customize','eos-dp' ) ),
				'query-attachments' => array( 'description' => __( 'Query attachments','eos-dp' ) )
			)
		),
		'all-in-one-wp-migration' => array(
			'is_active' => defined( 'AI1WM_PATH' ),
			'ajax_actions' => array(
				'ai1wm_import' => array( 'description' => __( 'installation importing','eos-dp' ),'notes' => __( 'Be careful, you should disable the unwanted plugins both on the source before the exporting and on the destination installation','eos-dp' ) ),
				'ai1wm_export' => array( 'description' => __( 'installation exporting','eos-dp' ) ),
				'ai1wm_status' => array( 'description' => __( 'checking status','eos-dp' ) )
			)
		),
		'woocommerce' => array(
			'is_active' => defined( 'WC_PLUGIN_FILE' ),
			'ajax_actions' => array(
				'add_to_cart' => array( 'description' => __( 'add to cart','eos-dp' ) ),
				'remove_from_cart' => array( 'description' => __( 'remove from cart','eos-dp' ) ),
				'apply_coupon' => array( 'description' => __( 'apply coupon','eos-dp' ) ),
				'remove_coupon' => array( 'description' => __( 'remove coupon','eos-dp' ) ),
				'checkout' => array( 'description' => __( 'checkout ajax refresh','eos-dp' ) ),
				'get_cart_totals' => array( 'description' => __( 'get cart totals','eos-dp' ) ),
				'get_customer_location' => array( 'description' => __( 'get customer location','eos-dp' ) ),
				'get_refreshed_fragments' => array( 'description' => __( 'get refreshed fragments','eos-dp' ) ),
				'get_variation' => array( 'description' => __( 'get variation','eos-dp' ) ),
				'update_order_review' => array( 'description' => __( 'update order review','eos-dp' ) ),
				'update_shipping_method' => array( 'description' => __( 'update shipping method','eos-dp' ) ),
			)
		),
		'wp-fastest-cache' => array(
			'is_active' => defined( 'WPFC_WP_CONTENT_BASENAME' ),
			'ajax_actions' => array(
				'wpfc_delete_cache' => array( 'description' => __( 'delete cache','eos-dp' ) ),
				'wpfc_delete_cache_and_minified' => array( 'description' => __( 'delete cache and minified','eos-dp' ) ),			)
		)
	);
	$plugins = apply_filters( 'eos_dp_integration_action_plugins',$plugins );
	foreach( $plugins as $plugin_slug => &$arr ){
		if( !isset( $args['is_active'] ) || !$arr['is_active'] ){
			unset( $arr[$plugin_slug] );
		}
	}
	return $plugins;
}

//Saved options preview action button
function eos_dp_saved_preview_button( $url,$target = '_blank' ){
	?>
	<a title="<?php _e( 'View page loading plugins according the saved options','eos-dp' ); ?>" class="eos-dp-view" href="<?php echo add_query_arg( 'show_disabled_plugins',md5( $_SERVER['REMOTE_ADDR'].( absint( time()/1000 ) ) ),esc_url( $url ) ); ?>" target="eos-dp-view-<?php echo esc_attr( $target ); ?>"><span class="dashicons dashicons-visibility"></span></a>
	<?php
}
//FDP debug button
function eos_dp_debug_button( $url,$target = '_blank' ){
	?>
	<a title="<?php _e( 'Check if the plugins are really disabled according to the saved options.','eos-dp' ); ?>" class="eos-dp-debug" href="#" data-url="<?php echo esc_attr( esc_url( $url ) ); ?>" onclick="eos_dp_debug_options(this);return false;"><span class="dashicons dashicons-editor-help"></span></a>
	<?php
	wp_nonce_field( 'eos_dp_debug_options', 'eos_dp_debug_options' );
}

//Return true if it's a plugins deactivation settings page
function eos_dp_is_deactivation_page(){
	return isset( $_GET['page'] ) && in_array( $_GET['page'],eos_dp_deactivation_pages() );
}
//Return array of plugins deactivation settings Pages
function eos_dp_deactivation_pages(){
	return apply_filters( 'eos_dp_deactivation_pages',array( 'eos_dp_menu','eos_dp_by_post_type','eos_dp_by_archive','eos_dp_by_term_archive','eos_dp_mobile','eos_dp_search','eos_dp_url','eos_dp_admin_url','eos_dp_admin','eos_dp_integration','eos_dp_hooks' ) );
}
//It saves multiple metadata given the $meta_key and an associative array of post IDs  and values
function eos_dp_save_multiple_metadata( $meta_key,$arr ){
	if( empty( $arr ) || '' === $meta_key ) return false;
	global $wpdb;
	$meta_key = esc_sql( $meta_key );
	$arr = json_decode( sanitize_text_field( json_encode( $arr ) ),true );
	$post_ids = esc_sql( implode( ',',array_map( 'absint',array_keys( $arr ) ) ) );
	$values = '';
	foreach( $arr as $id => $v ){
		$v = esc_sql( $v );
		$id = esc_sql( $id );
		$values .= '('.$id.',\''.$v.'\'),';
	}
	$values = rtrim( $values,',' );
    $result = $wpdb->update(
        $wpdb->postmeta,
        array(
            'post_id'      => $post_ids,
            'meta_key'   => $meta_key,
            'meta_value' => $values,
        ),
		$post_ids
    );
	return $result;
}

//It retrieves multiple metadata given the $meta_key and the array of post IDs
function eos_dp_get_multiple_metadata( $meta_key,$ids ){
	if( empty( $ids ) || '' === $meta_key ) return false;
	global $wpdb;
	if( is_array( $ids ) ){
		$ids = implode( ',',array_map( 'absint',$ids ) );
	}
	elseif( is_string( $ids ) ){
		$ids = implode( ',',array_map( 'absint',explode( ',',$ids ) ) );
	}
	$ids = esc_sql( $ids );
	$meta_key = esc_sql( $meta_key );
	$sql = "SELECT post_id,meta_value FROM $wpdb->postmeta WHERE post_id IN ($ids) AND meta_key='$meta_key';";
	return $wpdb->get_results( $sql,OBJECT );
}

//It returns the plugin name by its paths
function eos_dp_name_by_path( $path ){
	return ucwords( str_replace( '-',' ',dirname( $path ) ) );
}

//Check the privilegs what the user can do with FDP
function eos_dp_user_capabilities( $user = false ){
	if( !$user ){
		if( function_exists( 'wp_get_current_user' ) ){
			$user = wp_get_current_user();
		}
		else{
			return false;
		}
	}
	$opts = eos_dp_get_option( 'eos_dp_roles_manager' );
	$value = '';
	$roles = $other_admins = false;
	if( $opts && isset( $opts['fdp-roles-manager'] ) ){
		$opts = $opts['fdp-roles-manager'];
		if( '' !== $opts && isset( $user->roles ) ){
			$user_roles = $user->roles;
			$user_role = $user_roles[0];
			$value = $opts;
			$opts = json_decode( str_replace( '\\','',sanitize_text_field( $opts ) ),true );
			$roles = $opts['roles'];
			$other_admins = $opts['other_admins'];
			if( isset( $roles[$user_role] ) && 'administrator' !== $user_role ){
				$values = $roles[$user_role];
				return array(
					'global_settings' => $values[0] ? true : false,
					'single_settings' => $values[1] ? true : false,
					'see_plugin' => $values[2] ? true : false
				);
			}
			elseif( 'administrator' === $user_role && in_array( strtolower( $user->user_login ),array_keys( $other_admins ) ) ){
				$admin_email = eos_dp_get_option( 'admin_email' );
				$main_admin = $user->user_email === $admin_email;
				$values = !$main_admin && isset( $other_admins[strtolower( $user->user_login )] ) ? $other_admins[sanitize_key( strtolower( $user->user_login ) )] : array( true,true,true );
				return array(
					'global_settings' => $values[0] ? true : false,
					'single_settings' => $values[1] ? true : false,
					'see_plugin' => $values[2] ? true : false
				);
			}
		}
	}
	if( function_exists( 'current_user_can' ) ){
		return current_user_can( 'activate_plugins' ) ? array( 'global_settings' => true,'single_settings' => true,'see_plugin' => true ) : array( 'global_settings' => false,'single_settings' => false,'see_plugin' => false );
	}
	return false;
}

//Updte line of code in file_exists
function eos_dp_update_file_line( $file,$search,$replace ){
	$access_type = get_filesystem_method();
	if( $access_type === 'direct' ){
		$updated = false;
		$creds = request_filesystem_credentials( admin_url(), '', false, false, array() );
		if ( !WP_Filesystem( $creds ) ) return false;
		global $wp_filesystem;
		if( empty( $wp_filesystem ) ){
			require_once ( ABSPATH . '/wp-admin/includes/file.php' );
			WP_Filesystem();
		}
		$file_content = $wp_filesystem->get_contents( $file );
		if( false === strpos( $file_content,$search ) ) return;
		$upload_dirs = wp_upload_dir();
		if( !$wp_filesystem->is_dir( $upload_dirs['basedir'].'/FDP' ) ){
			$wp_filesystem -> mkdir( $upload_dirs['basedir'].'/FDP' );
		}
		if( !$wp_filesystem->is_dir( $upload_dirs['basedir'].'/FDP/backups' ) ){
			$wp_filesystem -> mkdir( $upload_dirs['basedir'].'/FDP/backups' );
		}
		if( !$wp_filesystem->is_dir( $upload_dirs['basedir'].'/FDP/backups/temp' ) ){
			$wp_filesystem -> mkdir( $upload_dirs['basedir'].'/FDP/backups/temp' );
		}
		$file_name = pathinfo( $file,PATHINFO_FILENAME );
		$dir_name = pathinfo( $file,PATHINFO_DIRNAME );
		$extension = pathinfo( $file,PATHINFO_EXTENSION );
		if( $file_content ){
			$backup_file = $upload_dirs['basedir'].'/FDP/backups/temp/'.$file_name.'-'.time().'.'.$extension;
			$copied = $wp_filesystem -> copy( $file,$backup_file,true );
			if( $copied ){
				$updated = $wp_filesystem->put_contents(
				  $file,
				  str_replace( $search,$replace,$file_content ),
				  FS_CHMOD_FILE
				);
				if( $updated ) $wp_filesystem -> delete( $backup_file );
			}
		}
	}
}
