<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}
//Callback for deactivate by archive settings page
function eos_dp_smoke_tests_callback(){
	if( !current_user_can( 'activate_plugins' ) ){
	?>
		<h2><?php _e( 'Sorry, you have not the right for this page','eos-dp' ); ?></h2>
		<?php
		return;
	}
	eos_dp_alert_plain_permalink();
	eos_dp_navigation();
	$active_plugins = eos_dp_active_plugins();
	$plugins = eos_dp_get_plugins();
	$plugins_data = array();
	?>
	<div id="eos-dp-plugin-info">
		<h2><?php _e( 'Smoke Tests on the last plugin versions','eos-dp' ); ?></h2>
		<div id="eos-dp-plugins-comparison-top"></div>
		<div class="eos-dp-tests-actions-wrp eos-dp-margin-top-48">
			<span id="eos-dp-collapse-all" class="button"><?php _e( 'Collapse all','eos-dp' ); ?><span class="dashicons dashicons-editor-contract"></span></span>
			<span id="eos-dp-expand-all" class="button"><?php _e( 'Expand all','eos-dp' ); ?><span class="dashicons dashicons-editor-expand"></span></span>
		</div>
		<?php
		foreach( $active_plugins as $plugin_path ){
			if( isset( $plugins[$plugin_path] ) ){
				$summaryHTML = '';
				$plugin_slug = sanitize_key( dirname( $plugin_path ) );
				$plugin_name = strtoupper( str_replace( '-',' ',$plugin_slug ) );
				if( !$summaryHTML || '' === $summaryHTML ){
					$tests_url = 'https://plugintests.com/plugins/'.$plugin_slug.'/latest';
					$response = wp_remote_get( esc_url( $tests_url ) );
					if( is_wp_error( $response ) || !isset( $response['headers'] ) ) continue;
					$body = wp_remote_retrieve_body( $response );
					if( !isset( $response['response'] ) ) continue;
					$response = $response['response'];
					if( !isset( $response['code'] ) ) continue;
					if( 200 === $response['code'] ){
						if( class_exists( 'DOMDocument' ) && class_exists( 'DomXPath' ) ){
							libxml_use_internal_errors( true );
							$dom = new DOMDocument();
							$dom->loadHTML( $body );
							if( null === $dom ) continue;
							$finder = new DomXPath( $dom );
							foreach( array( 
								'breadcrumb',
								'col-md-3',
								'col-md-4',
								'modal-body',
								'navbar-header',
								'modal-header',
								'modal-footer',
								'navbar-collapse',
								'well'
							) as $classname ){
								$nodes = $finder->query("//*[contains(@class, '$classname')]");				
								foreach( $nodes as $node ){
									$node->parentNode->removeChild( $node );
								}
							}
							$classname = 'container';
							$panels = $finder->query("//*[contains(@class, '$classname')]");				
							foreach( $panels as $panel ){
								$summaryHTML .= eos_dp_get_inner_html( $panel );
							}
							$rows = $finder->query("//*[@id='benchmark-details']//tr");
							$rowsN = $rows->length;
							$n = 1;
							$data = array();
							foreach( $rows as $row ){
								if( $n === $rowsN - 1 ){
									$cols = $row->getElementsByTagName( 'td' );
									$k = 0;
									foreach( $cols as $col ){
										if( $k > 0 && !in_array( $k,array( 1,2,4,5 ) ) ){
											$data[] = false !== strpos( $col->nodeValue,'-' ) ? __( 'Insignificant','eos-dp' ) : $col->nodeValue;
										}
										++$k;
									}
									$plugins_data[$plugin_slug] = $data;
								}
								++$n;
							}						
						}	
					}
				}
			?>
		<div class="postbox-container">
			<div id="eos-dp-st-<?php echo esc_attr( $plugin_slug ); ?>" class="eos-dp-plugin-info-section eos-dp-margin-top-48 postbox close">
				<span class="eos-dp-open-div eos-dp-toggle-div dashicons dashicons-arrow-down"></span>
				<span class="eos-dp-open-close eos-dp-toggle-div dashicons dashicons-arrow-up"></span>				
				<h2><?php echo esc_html( $plugin_name ); ?></h2>
				<div>
					<?php 
					if( 200 === $response['code'] ){ ?>
						<a target="_blank" rel="noopener" href="https://plugintests.com/plugins/<?php echo esc_attr( $plugin_slug ); ?>/latest"><img src="https://plugintests.com/plugins/<?php echo esc_attr( $plugin_slug ); ?>/wp-badge.svg" /></a>
					<?php 
					}
					else{
						?>
						<span><?php _e( 'Not found','eos-dp' ); ?></span>
						<?php
					}
					?>
				</div>
				<div class="eos-dp-plugin-test-summary eos-dp-margin-top-48">
				<?php
				if( $summaryHTML && '' !== $summaryHTML ){
					?>
					<?php echo eos_dp_plugininfo_kses( $summaryHTML ); ?>
					<span class="eos-dp-open-close eos-dp-toggle-div dashicons dashicons-arrow-up" style="margin-top:-10px"></span>
				<?php 
				}else{
					?>
					<div>
						<p><?php printf( __( 'Tests results of %s not found.','eos-dp' ),esc_html( $plugin_name ) ); ?></p>
						<p><?php printf( __( 'Maybe %s is a premium plugin, these tests are not available for premium plugins.','eos-dp' ),esc_html( $plugin_name ) ); ?></p>
					</div>
					<?php
				}
				?>
				</div>
			</div>
		</div>
			<?php 
			}
		}
		if( !empty( $plugins_data ) ){
		?>
		<div id="eos-dp-plugins-comparison" style="padding-top:48px;clear:both">
			<table class="table table-striped" id="benchmark-details-comparison">
				<thead>
					<tr>
						<th><?php _e( 'Plugin','eos-dp' ); ?></th>
						<th><?php _e( 'Load time change *','eos-dp' ); ?></th>
						<th><?php _e( 'Memory usage change','eos-dp' ); ?></th>
						<th><?php _e( 'Badges','eos-dp' ); ?></th>
					</tr>
				</thead>
				<tbody>
					<?php
					foreach( $plugins_data as $plugin_slug => $arr ){
					?>
					<tr>
						<td>
							<a href="#eos-dp-st-<?php echo esc_attr( $plugin_slug ); ?>" onclick="document.getElementById('eos-dp-st-<?php echo esc_js( esc_attr( $plugin_slug ) ); ?>').getElementsByClassName('eos-dp-open-div')[0].click();">
								<?php echo esc_html( strtoupper( str_replace( '-',' ',$plugin_slug ) ) ); ?>
							</a>
						</td>
						<?php
						$k = 0;
						foreach( $arr as $value ){
							?>
							<td><?php echo esc_html( $value ); ?></td>
							<?php							
							++$k;
						}
						?>
						<td>
							<div>
								<a target="_blank" rel="noopener" href="https://plugintests.com/plugins/<?php echo esc_attr( $plugin_slug ); ?>/latest"><img src="https://plugintests.com/plugins/<?php echo esc_attr( $plugin_slug ); ?>/wp-badge.svg"></a>
								<a target="_blank" rel="noopener" href="https://plugintests.com/plugins/<?php echo esc_attr( $plugin_slug ); ?>/latest"><img src="https://plugintests.com/plugins/<?php echo esc_attr( $plugin_slug ); ?>/php-badge.svg"></a>
							</div>
						</td>
					</tr>
					<?php
					}
					?>
				</tbody>
			</table>
			<p>*<?php _e( "Loading time due to PHP parsing and database queries. It doesn't take into account HTTP requests.","eos-dp" ); ?></p>
		</div>
		<script>
		var eos_comparison_table = document.getElementById('eos-dp-plugins-comparison');
		document.getElementById('eos-dp-plugins-comparison-top').innerHTML = eos_comparison_table.innerHTML;
		eos_comparison_table.parentNode.removeChild(eos_comparison_table);
		</script>
		<?php
		}
		?>
		<p><?php printf( __(  'Tests performed by %s%s%s','eos-dp' ),'<a href="https://plugintests.com/" target="_blank" rel="noopener" >','https://plugintests.com/','</a>' ); ?></p>
		<div id="eos-dp-go-to-top" class="hover right" style="margin:48px 6px 0 6px;z-index:999"><span title="<?php _e( 'Go to top','eos-dp' ); ?>" style="background:#fff;padding:10px" class="dashicons dashicons-arrow-up-alt"></span></div>
	</div>
	<?php
}

//Return node inner HTML
function eos_dp_get_inner_html( $node ) {
    $innerHTML= '';
    $children = $node->childNodes;
    foreach( $children as $child ){
        $innerHTML .= $child->ownerDocument->saveXML( $child );
    }
    return $innerHTML;
} 

//Return allowed HTML tags for plugin info
function eos_dp_plugininfo_kses( $html ){
	return wp_kses( str_replace( 'Get badge code','',$html ),array(
		'span' => array( 'class' => array( 'glyphicon','glyphicon-ok','language-json','hljs' ) ),
		'strong' => array(),
		'div' => array( 'class' => array( 'panel-heading' ) ),
		'code' => array(),
		'pre' => array(),
		'p' => array(),
		'small' => array(),
		'ul' => array(),
		'li' => array(),
		'h1' => array(),
		'h2' => array(),
		'h3' => array(),
		'table' => array(),
		'thead' => array(),
		'td' => array(),
		'tr' => array(),
		'th' => array( 'colspan' => array(),'rowspan' => array() ),
		'img' => array( 'src' => array() ),
	) );
}